@@ -7,7 +7,7 @@
 (custom-add-loads 'tools '("ediff"))
 (custom-add-loads 'ediff-hook '("ediff-init" "ediff-mult"))
 (custom-add-loads 'ediff-diff '("ediff-diff"))
-(custom-add-loads 'ediff-window '("ediff-wind"))
+(custom-add-loads 'ediff-window '("ediff-help" "ediff-wind"))
 (custom-add-loads 'frames '("ediff-wind"))
 (custom-add-loads 'ediff-ptch '("ediff-ptch"))
 (custom-add-loads 'ediff-merge '("ediff-init" "ediff-merg"))
