@@ -1,8 +1,8 @@
 ;;; ediff-merg.el --- merging utilities
 
-;; Copyright (C) 1994, 1995, 1996, 1997 Free Software Foundation, Inc.
+;; Copyright (C) 1994, 95, 96, 97, 98, 99, 2000, 01, 02 Free Software Foundation, Inc.
 
-;; Author: Michael Kifer <kifer@cs.sunysb.edu>
+;; Author: Michael Kifer <kifer@cs.stonybrook.edu>
 
 ;; This file is part of GNU Emacs.
 
@@ -21,6 +21,8 @@
 ;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 ;; Boston, MA 02111-1307, USA.
 
+;;; Commentary:
+
 ;;; Code:
 
 (provide 'ediff-merg)
@@ -80,7 +82,7 @@ STRING4
 (defcustom ediff-show-clashes-only nil
   "*If t, show only those diff regions where both buffers disagree with the ancestor.
 This means that regions that have status prefer-A or prefer-B will be
-skipped over.  Nil means show all regions."
+skipped over.  nil means show all regions."
   :type 'boolean
   :group 'ediff-merge
   )
@@ -120,7 +122,7 @@ Buffer B."
 	(combo-region "")
 	(err-msg
 	 "ediff-combination-pattern: Invalid format. Please consult the documentation")
-	diff-region region-delim region-spec)
+	region-delim region-spec)
 
     (if (< (length pattern-list) 5)
 	(error err-msg))
@@ -131,7 +133,7 @@ Buffer B."
       (or (and (stringp region-delim) (memq region-spec '(A B Ancestor)))
 	  (error err-msg))
 
-      (condition-case err
+      (condition-case nil
 	  (setq combo-region
 		(concat combo-region
 			region-delim "\n"
@@ -219,9 +221,10 @@ Buffer B."
       (setq state-of-merge (ediff-get-state-of-merge n))
 
       (if remerging
-	  (let ((reg-A (ediff-get-region-contents n 'A ediff-control-buffer))
-		(reg-B (ediff-get-region-contents n 'B ediff-control-buffer))
-		(reg-C (ediff-get-region-contents n 'C ediff-control-buffer)))
+	  ;;(let ((reg-A (ediff-get-region-contents n 'A ediff-control-buffer))
+	  ;;	(reg-B (ediff-get-region-contents n 'B ediff-control-buffer))
+	  ;;	(reg-C (ediff-get-region-contents n 'C ediff-control-buffer)))
+	  (let ()
 		
 	    ;; if region was edited since it was first set by default
 	    (if (or (ediff-merge-changed-from-default-p n)
@@ -305,7 +308,7 @@ Combining is done according to the specifications in variable
   (interactive "P")
   (setq n (if (numberp n) (1- n) ediff-current-difference))
   
-  (let (regA regB reg-combined)
+  (let (reg-combined)
     ;;(setq regA (ediff-get-region-contents n 'A ediff-control-buffer)
     ;;	  regB (ediff-get-region-contents n 'B ediff-control-buffer))
     ;;(setq reg-combined (ediff-make-combined-diff regA regB))
@@ -348,7 +351,8 @@ Combining is done according to the specifications in variable
 
 	(reverse delim-regs-list)
 	)))
-	
+
+(eval-when-compile (defvar state-of-merge)) ; dynamic var
 
 ;; Check if the non-preferred merge has been modified since originally set.
 ;; This affects only the regions that are marked as default-A/B or combined.
@@ -384,4 +388,4 @@ Combining is done according to the specifications in variable
 ;;; eval: (put 'ediff-with-current-buffer 'edebug-form-spec '(form body))
 ;;; End:
 
-;; ediff-merg.el ends here
+;;; ediff-merg.el ends here
